/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          neticmp.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          ICMP
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        Internet Control Message Protocol
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETICMP_C

#include "netmon.h"
#include "neticmp.h"

/*
 * GLOBALS
 */

static NETICMPSTAT   stat1;
static NETICMPSTAT   stat2;
static NETICMPSTAT  *pold;
static NETICMPSTAT  *pnew;

/*
 * kernel access
 */

static KVMLIST symlist[] =
{
    { "_icmpstat" },
    { NULL }
};
#define NSYM  (int)(sizeof(symlist)/sizeof( KVMLIST ))

static KVMOFST ofst_icmpstat;

/*
 * screen access
 */

#define CL1  (0)
#define CL1v (CL1+25)
#define CL2  (CL1+38)
#define CL2i (CL2+23)
#define CL2o (CL2+34)

MODFUNC modicmp = { IcmpInit, IcmpText, IcmpUpdate, CL2+46, 14 };
MODTEXT icmptext[] =
{
    {  0, CL1,  "PROTOCOL: ICMP" },
    {  0, CL1v, " COUNT_____" },
    {  0, CL2,  "CATEGORY_______________" },
    {  0, CL2i, " #RQSTS__IN" },
    {  0, CL2o, " #RQSTS_OUT" },

    /* general stats */

    {  1, CL1, "icmp error calls" },
    {  2, CL1, "no error:old IP tooshort" },
    {  3, CL1, "no error:old was ICMP" },
    {  4, CL1, "icmp_code out of range" },
    {  5, CL1, "packet < ICMP_MINLEN" },
    {  6, CL1, "bad checksum" },
    {  7, CL1, "calculated bound mismatch" },
    {  8, CL1, "number of responses" },

    /* i/o history per ICMP type */

    {  1, CL2, " 0-Echo reply" },
    {  2, CL2, " 3-destination unreachable" },
    {  3, CL2, " 4-source quench" },
    {  4, CL2, " 5-redirect" },
    {  5, CL2, " 8-echo request" },
    {  6, CL2, "11-time exceeded" },
    {  7, CL2, "12-parameter problem" },
    {  8, CL2, "13-time stamp request" },
    {  9, CL2, "14-time stamp reply" },
    { 10, CL2, "15-information request" },
    { 11, CL2, "16-information reply" },
    { 12, CL2, "17-address mask request" },
    { 13, CL2, "18-address mask reply" },
};
#define NTEXT (sizeof(icmptext)/sizeof(MODTEXT))

/* ---------------------- screen layout ----------------------------------------
PROTOCOL: ICMP            COUNT_____
icmp error calls          1234567890
no error:old IP tooshort  1234567890
no error:old was ICMP     1234567890
icmp_code out of range    1234567890
packet < ICMP_MINLEN      1234567890
bad checksum              1234567890
calculated bound mismatch 1234567890
number of responses       1234567890

CATEGORY_______________ #RQSTS__IN #RQSTS_OUT
Echo reply              1234567890 1234567890
destination unreachable 1234567890 1234567890
source quench           1234567890 1234567890
redirect                1234567890 1234567890
echo request            1234567890 1234567890
time exceeded           1234567890 1234567890
parameter problem       1234567890 1234567890
time stamp request      1234567890 1234567890
time stamp reply        1234567890 1234567890
information request     1234567890 1234567890
information reply       1234567890 1234567890
address mask request    1234567890 1234567890
address mask reply      1234567890 1234567890
----------------------------------------------------------------------------- */

/*
 * local prototypes
 */

void IcmpCnt1( int x, int y, int *new );

#define O1(_dx,_dy,_f)    IcmpCnt1(x+_dx,y+_dy, &pnew->icmpstat._f)
#define H1(_dx,_dy,_f,_n) IcmpCnt1(x+_dx,y+_dy, &pnew->icmpstat._f[_n])

/*****************************************************************
** NAME:        IcmpInit                                    NETMON
** SYNOPSIS:    void IcmpInit
**              (
**                void
**              )
** RELATED TO:  Icmp..()
** DESCRIPTION: initialise icmp stats
** REMARKS:     locate kernel vars
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IcmpInit( void )
{
    int r;

    memset( &stat1, 0, sizeof( stat1 ));
    memset( &stat2, 0, sizeof( stat2 ));
    pold = &stat1;
    pnew = &stat2;

    /*
     * locate the wanted symbols
     */

    KvmNlist( symlist, NSYM, __FILE__ );
    ofst_icmpstat = symlist[0].n_value;

    /*
     * initialise the new (is next old!) DB
     */

    KvmRead( ofst_icmpstat, &pnew->icmpstat, sizeof( struct icmpstat ));
}

/*****************************************************************
** NAME:        IcmpText                                    NETMON
** SYNOPSIS:    void IcmpText
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**              )
** RELATED TO:  Icmp..()
** DESCRIPTION: display icmp texts
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IcmpText( int x, int y )
{
    TermValue( icmptext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        IcmpUpdate                                  NETMON
** SYNOPSIS:    void IcmpUpdate
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**              )
** RELATED TO:  Icmp..()
** DESCRIPTION: display icmp values
** REMARKS:     read the kernel
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IcmpUpdate( int x, int y )
{
    NETICMPSTAT *swap;

    swap = pold;
    pold = pnew;
    pnew = swap;

    KvmRead( ofst_icmpstat, &pnew->icmpstat, sizeof( struct icmpstat ));

    O1( CL1v,  1, icps_error );
    O1( CL1v,  2, icps_oldshort );
    O1( CL1v,  3, icps_oldicmp );
    O1( CL1v,  4, icps_badcode );
    O1( CL1v,  5, icps_tooshort );
    O1( CL1v,  6, icps_checksum );
    O1( CL1v,  7, icps_badlen );
    O1( CL1v,  8, icps_reflect );

    H1( CL2i,  1, icps_inhist,   0 );
    H1( CL2i,  2, icps_inhist,   3 );
    H1( CL2i,  3, icps_inhist,   4 );
    H1( CL2i,  4, icps_inhist,   5 );
    H1( CL2i,  5, icps_inhist,   8 );
    H1( CL2i,  6, icps_inhist,  11 );
    H1( CL2i,  7, icps_inhist,  12 );
    H1( CL2i,  8, icps_inhist,  13 );
    H1( CL2i,  9, icps_inhist,  14 );
    H1( CL2i, 10, icps_inhist,  15 );
    H1( CL2i, 11, icps_inhist,  16 );
    H1( CL2i, 12, icps_inhist,  17 );
    H1( CL2i, 13, icps_inhist,  18 );

    H1( CL2o,  1, icps_outhist,  0 );
    H1( CL2o,  2, icps_outhist,  3 );
    H1( CL2o,  3, icps_outhist,  4 );
    H1( CL2o,  4, icps_outhist,  5 );
    H1( CL2o,  5, icps_outhist,  8 );
    H1( CL2o,  6, icps_outhist, 11 );
    H1( CL2o,  7, icps_outhist, 12 );
    H1( CL2o,  8, icps_outhist, 13 );
    H1( CL2o,  9, icps_outhist, 14 );
    H1( CL2o, 10, icps_outhist, 15 );
    H1( CL2o, 11, icps_outhist, 16 );
    H1( CL2o, 12, icps_outhist, 17 );
    H1( CL2o, 13, icps_outhist, 18 );
}

/*****************************************************************
** NAME:        IcmpCnt1                                    NETMON
** SYNOPSIS:    void IcmpCnt1
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**                int *new    pointer to value
**              )
** RELATED TO:  Icmp..()
** DESCRIPTION: output ICMP stat counts
** REMARKS:     output formatter
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IcmpCnt1( int x, int y, int *new )
{
    unsigned long value = *new;
    tprintf( x, y, " % 10lu", value );
}

/* end of neticmp.c */


