/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netip.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          IP
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        Internet protocol stats retrieval
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETIP_C

#include "netmon.h"
#include "netip.h"

/*
 * GLOBALS
 */

static NETIPSTAT   stat1;
static NETIPSTAT   stat2;
static NETIPSTAT  *pold;
static NETIPSTAT  *pnew;

/*
 * kernel access
 */

static KVMLIST symlist[] =
{
    { "_ipstat" },            /* regular IP stat struct          */
    { "_ip_id" },             /* IP output packet sequence ID nr */
    { NULL }
};
#define NSYM  (int)(sizeof(symlist)/sizeof( KVMLIST ))

static KVMOFST ofst_ipstat;
static KVMOFST ofst_ip_id;

/*
 * screen access
 */

#define CL1  (0)
#define CL1v (CL1+13)

MODFUNC modip = { IpInit, IpText, IpUpdate, 35, 14};
MODTEXT iptext[] =
{
    {  0, CL1, "PROTOCOL : IP" },
    {  0, CL1v," #PACKETS__ #PCKT/SEC" },

    {  1, CL1, "received" },
    {  2, CL1, "  badsum" },
    {  3, CL1, "  tooshort" },
    {  4, CL1, "  toosmall" },
    {  5, CL1, "  badhdrlen" },
    {  6, CL1, "  bad length" },
    {  7, CL1, "  fragments" },
    {  8, CL1, "  fragdropped" },
    {  9, CL1, "  fragtimeout" },
    { 10, CL1, "  forwarded" },
    { 11, CL1, "  cantforward" },
    { 12, CL1, "  redirected" },
    { 13, CL1, "sent" }
};
#define NTEXT (int)(sizeof(iptext)/sizeof(MODTEXT))

/* -----------------------------------------------------------------------------
PROTOCOL : IP #PACKETS__ #PCKT/SEC
received      1234567890 123456.89
  badsum      1234567890 123456.89
  tooshort    1234567890 123456.89
  toosmall    1234567890 123456.89
  badhdrlen   1234567890 123456.89
  bad length  1234567890 123456.89
  fragments   1234567890 123456.89
  fragdropped 1234567890 123456.89
  fragtimeout 1234567890 123456.89
  forwarded   1234567890 123456.89
  cantforward 1234567890 123456.89
  redirected  1234567890 123456.89
sent          1234567890 123456.89
----------------------------------------------------------------------------- */

/*
 * local prototypes
 */

void IpCnt( int x, int y, long *new, long *old);

#define Oul(_dx,_dy,_fld) IpCnt(x+_dx,y+_dy,\
                                &pnew->ipstat._fld,\
                                &pold->ipstat._fld)

#define Ous(_dx,_dy,_fld) IpCnt(x+_dx,y+_dy,\
                                &pnew->_fld,\
                                &pold->_fld)

/*****************************************************************
** NAME:        IpInit                                      NETMON
** SYNOPSIS:    void IpInit
**              (
**                void
**              )
** RELATED TO:  Ip..()
** DESCRIPTION: initialise ip stats
** REMARKS:     locate kernel vars
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IpInit( void )
{
    int r;

    memset( &stat1, 0, sizeof( stat1 ));
    memset( &stat2, 0, sizeof( stat2 ));
    pold = &stat1;
    pnew = &stat2;

    /*
     * locate the wanted symbols
     */

    KvmNlist( symlist, NSYM, __FILE__ );
    ofst_ipstat = symlist[0].n_value;
    ofst_ip_id  = symlist[1].n_value;

    /*
     * initialise the new (is next old!) DB
     */

    KvmRead( ofst_ipstat, &pnew->ipstat, sizeof( struct ipstat ));
    KvmRead( ofst_ip_id,  &pnew->id,     sizeof( u_short ));
    pnew->snd_total = pnew->id;
}

/*****************************************************************
** NAME:        IpText                                      NETMON
** SYNOPSIS:    void IpText
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**              )
** RELATED TO:  Ip..()
** DESCRIPTION: display ip texts
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IpText( int x, int y )
{
    TermValue( iptext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        IpUpdate                                    NETMON
** SYNOPSIS:    void IpUpdate
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**              )
** RELATED TO:  Ip..()
** DESCRIPTION: display ip values
** REMARKS:     read the kernel vars
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IpUpdate( int x, int y )
{
    u_short    sent;
    NETIPSTAT *swap;

    swap = pold;
    pold = pnew;
    pnew = swap;

    KvmRead( ofst_ipstat, &pnew->ipstat, sizeof( struct ipstat ));
    KvmRead( ofst_ip_id,  &pnew->id,     sizeof( u_short ));

    Oul( CL1v,  1, ips_total );
    Oul( CL1v,  2, ips_badsum );
    Oul( CL1v,  3, ips_tooshort );
    Oul( CL1v,  4, ips_toosmall );
    Oul( CL1v,  5, ips_badhlen );
    Oul( CL1v,  6, ips_badlen );
    Oul( CL1v,  7, ips_fragments );
    Oul( CL1v,  8, ips_fragdropped );
    Oul( CL1v,  9, ips_fragtimeout );
    Oul( CL1v, 10, ips_forward );
    Oul( CL1v, 11, ips_cantforward );
    Oul( CL1v, 12, ips_redirectsent );

    /*
     * id field is a u_short and may wrap!!
     */

    if ( pnew->id < pold->id )
        sent = pold->id - pnew->id;
    else
        sent = pnew->id - pold->id;

    pnew->snd_total = pold->snd_total + (long) sent;
    Ous( CL1v, 13, snd_total );
}

/*****************************************************************
** NAME:        IpCnt                                       NETMON
** SYNOPSIS:    void IpCnt
**              (
**                int x,      relative column origin
**                int y,      relative row    origin
**                long *new,  pointer to new value
**                long *old   pointer to old value
**              )
** RELATED TO:  Ip..()
** DESCRIPTION: output IP stat counts
** REMARKS:     output formatter
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void IpCnt( int x, int y, long *new, long *old)
{
    tprintf( x, y, " % 10lu % 9.2f",
                   *new,
                   (double) ((double) (*new-*old)/timedone));
}

/* end of netip.c */


