/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netkern.h
** SYSTEM NAME:        Netmon
** TYPE NAME:          kernel access
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        header file for all Netmon stat modules
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#ifndef _NETKERN_H
#define _NETKERN_H

#define KERNEL

/*
 * extra includes
 */

#include <sys/types.h>
#include <unistd.h>
#include <kvm.h>
#include <fcntl.h>
#include <nlist.h>
#include <sys/time.h>

/*
 * typedef,defines, ...
 */

#define VALID   (1)
#define INVALID (0)

typedef struct nlist  KVMLIST;
typedef unsigned long KVMOFST;

typedef void (*MOD_INIT)(void);
typedef void (*MOD_DRAWTEXT)(int, int);
typedef void (*MOD_UPDATE)(int, int);

typedef struct modfunctions MODFUNC;
struct modfunctions
{
   MOD_INIT     init;
   MOD_DRAWTEXT text;
   MOD_UPDATE   update;
   int          xdim;         /* partial screen width */
   int          ydim;         /* partial screen height */
};

/*
 * externs
 */

#ifndef _NETKERN_C
extern kvm_t  *kvmH;
extern double  timebegin;
extern double  timenew;
extern double  timeold;
extern double  timedone;
extern double  kvmreadbytes;
extern long    kvmreaderrors;
#endif

extern int errno;
extern int sys_nerr;
extern int sys_errlist[];

/*
 * prototypes
 */

void   KvmOpen ( void );
void   KvmClose( void );
void   KvmRead ( KVMOFST addr, void *buf, unsigned nbytes );
int    KvmRead2( KVMOFST addr, void *buf, unsigned nbytes );
int    KvmRead3( KVMOFST addr, void *buf, unsigned nbytes );
void   KvmNlist( struct nlist *nl, int length, char *name );

void   TimeOpen( void );
double TimeRead( void );
void   TimeGet ( void );

#endif  /* end of netkern.h */











