/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          netmem.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          mbufs
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        mbuf related network statistics
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETMEM_C

#include "netmon.h"
#include "netmem.h"

/*
 * GLOBALS
 */

static NETMEMSTAT   stat1;
static NETMEMSTAT   stat2;
static NETMEMSTAT  *pold;
static NETMEMSTAT  *pnew;

/*
 * kernel access
 */

static KVMLIST symlist[] =
{
    { "_mbstat" },
    { NULL }
};
#define NSYM  (int)(sizeof(symlist)/sizeof( KVMLIST ))

static KVMOFST ofst_memstat;

/*
 * screen access
 */

#define CL1  (0)
#define CL1v (CL1+39)
#define CL2  (CL1+52)
#define CL2v (CL2+26)

MODFUNC modmem = { MemInit, MemText, MemUpdate, CL2+38, 16};
MODTEXT memtext[] =
{
    {  0, CL1, "MBUF_GENERAL_STATS_____________________" },
    {  0, CL1v," #COUNTERS_" },

    {  0, CL2, "MBUF_USAGE________________" },
    {  0, CL2v," #COUNTERS_" },

    /* general stats */

    {  1, CL1, "mbufs obtained from page pool" },
    {  2, CL1, "clusters obtained from page pool" },
    {  3, CL1, "free clusters" },
    {  4, CL1, "times failed to find space" },
    {  5, CL1, "interface pages obtained from page pool" },
    {  6, CL1, "times waited for space" },
    {  7, CL1, "times drained protocols for space" },

    {  1, CL2, "should be on free list" },
    {  2, CL2, "dynamic (data) allocation" },
    {  3, CL2, "packet header" },
    {  4, CL2, "socket structure" },
    {  5, CL2, "protocol control block" },
    {  6, CL2, "routing tables" },
    {  7, CL2, "IMP host tables" },
    {  8, CL2, "address resolution tables" },
    {  9, CL2, "socket name" },
    { 10, CL2, "zombie proc status" },
    { 11, CL2, "socket options" },
    { 12, CL2, "fragment reassembly header" },
    { 13, CL2, "access rights" },
    { 14, CL2, "interface address" }
};
#define NTEXT (sizeof(memtext)/sizeof(MODTEXT))

/* -----------------------------------------------------------------------------
MBUF_GENERAL_STATS_____________________ #COUNTERS_
mbufs obtained from page pool           1234567890
clusters obtained from page pool        1234567890
free clusters                           1234567890
times failed to find space              1234567890
interface pages obtained from page pool 1234567890
times waited for space                  1234567890
times drained protocols for space       1234567890

MBUF_USAGE________________ #COUNTERS_
should be on free list     1234567890
dynamic (data) allocation  1234567890
packet header              1234567890
socket structure           1234567890
protocol control block     1234567890
routing tables             1234567890
IMP host tables            1234567890
address resolution tables  1234567890
socket name                1234567890
zombie proc status         1234567890
socket options             1234567890
fragment reassembly header 1234567890
access rights              1234567890
interface address          1234567890
----------------------------------------------------------------------------- */

/*
 * local prototypes
 */

void MemCnt1( int x, int y, u_long *new );
void MemCnt1s( int x, int y, short *new );

#define O1(_dx,_dy,_f)     MemCnt1(x+_dx,y+_dy,  &pnew->memstat._f)
#define O1S(_dx,_dy,_f)    MemCnt1s(x+_dx,y+_dy, &pnew->memstat._f)
#define H1S(_dx,_dy,_f,_n) MemCnt1s(x+_dx,y+_dy, &pnew->memstat._f[_n])

/*****************************************************************
** NAME:        MemInit                                     NETMON
** SYNOPSIS:    void MemInit
**              (
**                void
**              )
** RELATED TO:  Mem..()
** DESCRIPTION: initialise mem stats
** REMARKS:     locate kernel vars
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void MemInit( void )
{
    int r;

    memset( &stat1, 0, sizeof( stat1 ));
    memset( &stat2, 0, sizeof( stat2 ));
    pold = &stat1;
    pnew = &stat2;

    /*
     * locate the wanted symbols
     */

    KvmNlist( symlist, NSYM, __FILE__ );
    ofst_memstat = symlist[0].n_value;

    /*
     * initialise the new (is next old!) DB
     */

    KvmRead( ofst_memstat, &pnew->memstat, sizeof( struct mbstat ));
}

/*****************************************************************
** NAME:        MemText                                     NETMON
** SYNOPSIS:    void MemText
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**              )
** RELATED TO:  Mem..()
** DESCRIPTION: display mem texts
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void MemText( int x, int y )
{
    TermValue( memtext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        MemUpdate                                   NETMON
** SYNOPSIS:    void MemUpdate
**              (
**                int x,      relative column origin
**                int y       relative row    origin
**              )
** RELATED TO:  Mem..()
** DESCRIPTION: display mem values
** REMARKS:     read the kernel
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void MemUpdate( int x, int y )
{
    NETMEMSTAT *swap;

    swap = pold;
    pold = pnew;
    pnew = swap;

    KvmRead( ofst_memstat, &pnew->memstat, sizeof( struct mbstat ));

    O1S( CL1v,  1, m_mbufs );
    O1S( CL1v,  2, m_clusters );
    O1S( CL1v,  3, m_clfree );
    O1S( CL1v,  4, m_drops );
    O1(  CL1v,  5, m_space );
    O1(  CL1v,  6, m_wait );
    O1(  CL1v,  7, m_drain );

    H1S( CL2v,  1, m_mtypes, MT_FREE   );
    H1S( CL2v,  2, m_mtypes, MT_DATA   );
    H1S( CL2v,  3, m_mtypes, MT_HEADER );
    H1S( CL2v,  4, m_mtypes, MT_SOCKET );
    H1S( CL2v,  5, m_mtypes, MT_PCB    );
    H1S( CL2v,  6, m_mtypes, MT_RTABLE );
    H1S( CL2v,  7, m_mtypes, MT_HTABLE );
    H1S( CL2v,  8, m_mtypes, MT_ATABLE );
    H1S( CL2v,  9, m_mtypes, MT_SONAME );
    H1S( CL2v, 10, m_mtypes, MT_ZOMBIE );
    H1S( CL2v, 11, m_mtypes, MT_SOOPTS );
    H1S( CL2v, 12, m_mtypes, MT_FTABLE );
    H1S( CL2v, 13, m_mtypes, MT_RIGHTS );
    H1S( CL2v, 14, m_mtypes, MT_IFADDR );
}

/*****************************************************************
** NAME:        MemCnt1                                     NETMON
** SYNOPSIS:    void MemCnt1
**              (
**                int x,      relative column origin
**                int y,      relative row    origin
**                u_long *new pointer to value
**              )
** RELATED TO:  Mem..()
** DESCRIPTION: output MEM stat counts
** REMARKS:     output formatter
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void MemCnt1( int x, int y, u_long *new )
{
    tprintf( x, y, " % 10lu", *new );
}

/*****************************************************************
** NAME:        MemCnt1s                                    NETMON
** SYNOPSIS:    void MemCnt1s
**              (
**                int x,      relative column origin
**                int y,      relative row    origin
**                short *new  pointer to value
**              )
** RELATED TO:  Mem..()
** DESCRIPTION: output MEM stat counts
** REMARKS:     output formatter
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void MemCnt1s( int x, int y, short *new )
{
    tprintf( x, y, " % 10u", (u_short) *new );
}

/* end of netmem.c */


