/************************************************************************
** MODULE INFORMATION **
************************
** FILE NAME:          nettcpso.c
** SYSTEM NAME:        Netmon
** TYPE NAME:          network TCP sockets
** ORIGINAL AUTHOR(S): L. Th. Weerpas & W. Haverhals
** VERSION NUMBER:     1.0
** CREATION DATE:      1992/6/9
**
** DESCRIPTION:        network TCP socket info retrieval
**                     This is one of most interesting modules of XNetmon
**                     and is able to show per TCP socket dynamic info.
*************************************************************************
** CHANGES INFORMATION **
*************************
** REVISION:           $Revision$
** WORKFILE:           $Workfile$
** LOGINFO:            $Log$
*************************************************************************
** COPYRIGHT           (c) 1992
**                     L. Th. Weerpas & W. Haverhals
**                     All rights reserved.
**
** NOTICE              Redistribution  and use in source and binary forms
**                     are  permitted  provided  that the above copyright
**                     notice  and  this  paragraph are duplicated in all
**                     such forms and that any documentation, advertising
**                     materials,  and  other  materials  related to such
**                     distribution and use acknowledge that the software
**                     was  developed  by  above  mentioned  authors. The
**                     name(s)  of the authors may not be used to endorse
**                     or  promote  products  derived  from this software
**                     without  specific  prior  written permission.
**                     THIS  SOFTWARE  IS  PROVIDED ``AS IS'' AND WITHOUT
**                     ANY  EXPRESS  OR  IMPLIED  WARRANTIES,  INCLUDING,
**                     WITHOUT  LIMITATION,  THE  IMPLIED  WARRANTIES  OF
**                     MERCHANTIBILITY   AND  FITNESS  FOR  A  PARTICULAR
**                     PURPOSE.
************************************************************************/

#define _NETTCPSO_C

#define  TCPSTATES             /* see <netinet/tcp_fsm.h> */

#include "netmon.h"
#include "nettcp.h"
#include "nettcpso.h"

/*
 * GLOBALS
 */

static char header[] = "STATE_______  "
                       "REMOTE_ADDRESS_/_PORT__  LOCAL_ADDRESS__/_PORT__  "
                       "BYTE_SENT_  BYTE_RCVD_  "
                       "SOCK_OPTIONS____  "
                       "RBFmx_  SBFmx_  "
                       "RBF___  SBF___  "
                       "USERNAME__  PROCESS_________";
#define NHDR (sizeof(header))

MODFUNC modtcpso = { TcpSoInit,TcpSoText,TcpSoUpdate, NHDR, MTS+1 };
MODTEXT tcpsotext[] =
{
    { 0,  0, header }
};
#define NTEXT (sizeof(tcpsotext)/sizeof(MODTEXT))

/*
 * kernel access
 */

static KVMLIST symlist[] =
{
    { "_tcb" },
    { NULL }
};
#define NSYM  (int)(sizeof(symlist)/sizeof( KVMLIST ))

static KVMOFST     ofst_tcb;

/*
 * available tcb's
 */




static short       npcbs;               /* nr of inpcb's for TCP */
static NETTCPSO    tcpso[ MAX_SO ];
static struct user usr;                 /* temp user area        */

static NETTCPSO *root[ TCP_NSTATES ];
static NETTCPSO *last[ TCP_NSTATES ];
static char      username[ 256 ];
static char      processname[ 256 ];

/*
 * local prototypes
 */

void TcpSoStateInfo( int x, int *y, int *linecount, int state );
void TcpSoInfo( int x, int *y, int * linecount, NETTCPSO *ptis );

/*****************************************************************
** NAME:        TcpSoInit                                   NETMON
** SYNOPSIS:    void TcpSoInit
**              (
**                void
**              )
** RELATED TO:  TcpSo..()
** DESCRIPTION: initialise tcpso stats
** REMARKS:     - should run under extreme priority:
**                equal or larger than the network software!!
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpSoInit( void )
{
    /*
     * init
     */

    memset( &tcpso, 0, sizeof( tcpso ));

    /*
     * locate the wanted interface link root
     */

    KvmNlist( symlist, NSYM, __FILE__ );
    ofst_tcb = symlist[0].n_value;
}

/*****************************************************************
** NAME:        TcpSoText                                   NETMON
** SYNOPSIS:    void TcpSoText
**              (
**                int x,   absolute column origin
**                int y    absolute row    origin
**              )
** RELATED TO:  TcpSo..()
** DESCRIPTION: display tcpso texts
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpSoText( int x, int y )
{
    TermValue( tcpsotext, x, y, NTEXT );
}

/*****************************************************************
** NAME:        TcpSoUpdate                                 NETMON
** SYNOPSIS:    void TcpSoUpdate
**              (
**                int x,   absolute column origin
**                int y    absolute row    origin
**              )
** RELATED TO:  TcpSo..()
** DESCRIPTION: display tcpso values
** REMARKS:     - each time this routine is called it completely
**                fetches all the TCP pcb and related data
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpSoUpdate( int x, int y )
{
    int       r;              /* result of KvmRead2() */
    int       row;
    int       cnt;
    int       s, i;

    KVMOFST   ofst_inpcb;
    KVMOFST   ofst_sock;
    KVMOFST   ofst_tcpcb;
    KVMOFST   ofst_rprc;
    KVMOFST   ofst_sprc;
    NETTCPSO *ptis;
    INPCB    *pin;
    SOCKET   *pso;
    PROCESS  *pprc;
    TCPCB    *ptcb;

    /*
     * Run  this  routine  on  a very high priority level. We
     * must and I mean MUST ensure that the network subsystem
     * DOES  NOT change the linked lists while we are reading
     * them.   This  could  cause  incorrect  data  and  also
     * segmentation violations.
     */

    row = y + 1;
    cnt = MTS;

    s = spl3();

    /*
     * Retrieve all the data structures. We only retrieve TCP
     * connections   with   existing   SOCKETs   and   TCPCBs
     */

    ofst_inpcb = ofst_tcb;                     /* first entry offset */

    for ( npcbs=0; npcbs < MAX_SO; )
    {
        ptis = &tcpso[ npcbs ];

        /*
         * read INPCB : '_tcb' is a struct and thus always present
         */

        r = KvmRead2( ofst_inpcb, &ptis->ipcb, sizeof( INPCB ));
        if ( r < 0 )
            break;

        /*
         * read SOCKET : possibly present
         */

        ofst_sock = (KVMOFST) ptis->ipcb.inp_socket;
        if ( ofst_sock )
        {
            r = KvmRead2( ofst_sock, &ptis->sock, sizeof( SOCKET ));
            if ( r < 0 )
                goto L0;
            ptis->sockok = VALID;

            /*
             * read receiving PROCESS
             */

            ofst_rprc = (KVMOFST) ptis->sock.so_rcv.sb_sel;
            r = KvmRead2( ofst_rprc, &ptis->rprc, sizeof( PROCESS ));
            if ( r >= 0 )
                ptis->rprocok = VALID;

            /*
             * read sending PROCESS
             */

            ofst_sprc = (KVMOFST) ptis->sock.so_snd.sb_sel;
            r = KvmRead2( ofst_sprc, &ptis->sprc, sizeof( PROCESS ));
            if ( r >= 0 )
                ptis->sprocok = VALID;
        }
        else
            goto L0;

        /*
         * read TCP tcb : possibly present
         */

        ofst_tcpcb = (KVMOFST) ptis->ipcb.inp_ppcb;
        if ( ofst_tcpcb )
        {
            r = KvmRead2( ofst_tcpcb, &ptis->ttcb, sizeof( TCPCB ));
            if ( r < 0 )
                goto L0;
            ptis->tcbok = VALID;
        }
        else
            goto L0;

        /*
         * done, to the next inpcb
         */

        ++npcbs;  /* we have found a new TCP connection */

L0:     ;
#ifndef  SORTED
        TcpSoInfo( x, &row, &cnt, ptis );
#endif
        ofst_inpcb = (KVMOFST) ptis->ipcb.inp_next;
        if ( ofst_inpcb == ofst_tcb )
            break;
    }

    /*
     * reset the priority
     */

     splx(s);

    /*
     * sort the entries on TCP states, see tcp_fsm.h
     */

#ifndef SORTED
    return;
#endif

    memset( root, 0, sizeof( root ));             /* begin of TCPS_? chain */
    memset( last, 0, sizeof( last ));             /* last in  TCPS_? chain */

    /* build the chains off TCP states */
    /* fastest method !  O(npcbs)      */

    row = y + 1;
    cnt = MTS;

    for ( i=0; i < npcbs; i++ )
    {
        short state;

        ptis  = &tcpso[ i ];

        if ( ptis->tcbok == INVALID )
            continue;

        state = ptis->ttcb.t_state;
        if ( root[ state ] == NULL )
        {
            root[ state ] = ptis;
            last[ state ] = ptis;
        }
        else
        {
            last[ state ]->next = ptis;
            last[ state ] = ptis;
        }
    }

    /* end all chains, if any */

    for ( i=0; i < TCP_NSTATES; i++ )
    {
        if ( last[ i ] != NULL )
            last[ i ]->next = NULL;
    }

    /*
     * update the terminal screen, traverse the state chains.
     */

    row = y + 1;
    cnt = MTS;

    TcpSoStateInfo( x, &row, &cnt, TCPS_ESTABLISHED );      /* active        */
    TcpSoStateInfo( x, &row, &cnt, TCPS_TIME_WAIT );        /* to be removed */
    TcpSoStateInfo( x, &row, &cnt, TCPS_CLOSE_WAIT );
    TcpSoStateInfo( x, &row, &cnt, TCPS_LISTEN );           /* passive       */
    TcpSoStateInfo( x, &row, &cnt, TCPS_SYN_SENT );
    TcpSoStateInfo( x, &row, &cnt, TCPS_SYN_RECEIVED );
    TcpSoStateInfo( x, &row, &cnt, TCPS_FIN_WAIT_1 );
    TcpSoStateInfo( x, &row, &cnt, TCPS_CLOSING );
    TcpSoStateInfo( x, &row, &cnt, TCPS_LAST_ACK );
    TcpSoStateInfo( x, &row, &cnt, TCPS_FIN_WAIT_2 );
    TcpSoStateInfo( x, &row, &cnt, TCPS_CLOSED );

    if ( cnt )
    {
        TermSetXY( 0,row );
        TermClearToEnd();
    }
    return;
}

/*****************************************************************
** NAME:        TcpSoStateInfo                              NETMON
** SYNOPSIS:    void TcpSoStateInfo
**              (
**                int  x,         absolute column origin
**                int *y,         absolute row    origin
**                int *linecount, nr of lines left on screen
**                int  state      TCP state value
**              )
** RELATED TO:  TcpSo..()
** DESCRIPTION: display tcpso values
** REMARKS:     - display the info of 1 TCP state chain, beginning
**                at 'root[state]'
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpSoStateInfo( int x, int *y, int *linecount, int state )
{
    NETTCPSO *ptis;

    /*
     * no more ??
     */

    if ( *linecount <= 0 )
        return;

    /*
     * traverse the chain
     */

    for ( ptis = root[ state ]; ptis != NULL; ptis = ptis->next )
    {
         TcpSoInfo( x, y, linecount, ptis );
    }
    return;
}

/*****************************************************************
** NAME:        TcpSoInfo                                   NETMON
** SYNOPSIS:    void TcpSoInfo
**              (
**                int  x,         absolute column origin
**                int *y,         absolute row    origin
**                int *linecount, nr of lines left on screen
**                NETTCPSO *ptis  pointer to TCP conn state struct
**              )
** RELATED TO:  TcpSo..()
** DESCRIPTION: display contents of 1 TCP inpcb
** REMARKS:     -
** RETURN TYPE: void
** EXIT STATUS: -
*****************************************************************/

void TcpSoInfo( int x, int *y, int *linecount, NETTCPSO *ptis )
{
    INPCB    *pin;
    SOCKET   *pso;
    PROCESS  *prprc;
    PROCESS  *psprc;
    TCPCB    *ptcb;
    tcp_seq   sent;
    tcp_seq   rcvd;

    /*
     * no more ??
     */

    if ( (*linecount) <= 0 )
        return;

        /*
         * set the pointers
         */

        pin   = &ptis->ipcb;
        pso   = &ptis->sock;
        ptcb  = &ptis->ttcb;

        /*
         * 14   connection state
         */

        TermSetXY( x, (*y)++ );

        if ( ptcb->t_state < TCP_NSTATES )
            cprintf( "%12.12s  ", tcpstates[ ptcb->t_state ] );
        else
            cprintf( "<bad state!>  " );

        /*
         * 50   connection addresses
         */

        SockAdrPortOut( &pin->inp_faddr, pin->inp_fport, NET_TCP );
        SockAdrPortOut( &pin->inp_laddr, pin->inp_lport, NET_TCP );

        /*
         * 24   nr of transferred bytes
         */

        if ( ptis->tcbok == VALID )
        {
            if ( ptcb->t_state == TCPS_ESTABLISHED )
            {
                if ( ptcb->snd_una >= ptcb->iss )
                    sent = ptcb->snd_una - ptcb->iss;
                else
                    sent = ptcb->iss - ptcb->snd_una;

                if ( ptcb->rcv_nxt >= ptcb->irs )
                    rcvd = ptcb->rcv_nxt - ptcb->irs;
                else
                    rcvd = ptcb->irs - ptcb->rcv_nxt;

                cprintf( "% 10lu  % 10lu  ", sent, rcvd );
            }
            else
            {
                cprintf( "----------  ----------  " );
            }
        }
        else
        {
            cprintf( "---bad----  ---tcb----  " );
        }

        /*
         * 22   socket, user and  process
         */

        if ( ptis->sockok == VALID )
        {
             SockOptions( pso );
             SockBufSize( pso );

             if ( ptis->rprocok == VALID )
             {
                cprintf( "%10.10s  %16.16s",
                         GetUserName( ptis->rprc.p_uid, username   ),
                         GetProcName( &ptis->rprc, processname ) );
             }
             else if ( ptis->sprocok == VALID )
             {
                cprintf( "%10.10s  %16.16s",
                         GetUserName( ptis->sprc.p_uid, username   ),
                         GetProcName( &ptis->sprc, processname ) );
             }
             else
                cprintf( "- user? --  --- process? ---" );
        }
        else
        {
            cprintf( "                  " );
            cprintf( "              " );
            cprintf( "- user? --  --- process? ---" );
        }

        /*
         *>104   change linecount: end of screen reached !?
         */

        (*linecount)--;
        if ( (*linecount) <= 0 )
            return;
}

/*****************************************************************
** NAME:        GetUserName                                 NETMON
** SYNOPSIS:    char *GetUserName
**              (
**                int   uid,   our user's ID
**                char *buf    buffer to receive the username
**              )
** RELATED TO:  TcpSo..()
** DESCRIPTION: resolve a user name
** REMARKS:     -
** RETURN TYPE: char *
** EXIT STATUS: 'buf'
*****************************************************************/

char *GetUserName( int uid, char *buf )
{
    char          *tmp = buf;
    struct passwd *pwd;

    pwd = getpwuid( uid );
    if ( pwd == NULL )
        sprintf( buf, "% 10ld", (unsigned long) uid );      /* numeric UID */
    else
        sprintf( buf, "%10s", pwd->pw_name );               /* real name   */
    return tmp;
}

/*****************************************************************
** NAME:        GetProcName                                 NETMON
** SYNOPSIS:    char *GetProcName
**              (
**                PROCESS *proc   our process struct
**                char    *buf    buffer to receive the username
**              )
** RELATED TO:  TcpSo..()
** DESCRIPTION: resolve a process name
** REMARKS:     - pid's 0,1,2 are translated
** RETURN TYPE: char *
** EXIT STATUS: 'buf'
*****************************************************************/

char *GetProcName( PROCESS *proc, char *buf )
{
    struct user *pusr;

    if ( proc == NULL )
    {
        *buf = '\0';
        return buf;
    }

    /*
     * get the user area
     */

    pusr = kvm_getu( kvmH, proc ) ;
    if ( pusr == NULL )
    {
        strcpy( buf, "<swapped out>" );
        return buf;
    }
    memcpy( &usr, pusr, sizeof( usr));

    if ( usr.u_comm[ 0 ] == '\0' )
    {
        switch ( proc->p_pid )
        {
            case 0:                              /* highly unlikely processes */
                strcpy( buf, "<SWAPPER>" );      /* to use networking !!      */
                break;
            case 2:
                strcpy( buf, "<PAGER>" );
                break;
            default:
                strcpy( buf, "<?>" );
                break;
        }
    }
    else
    {
        strncpy( buf, usr.u_comm, MAXCOMLEN + 1 );
        buf[ MAXCOMLEN ] = '\0';
    }
    return buf;
}

/*****************************************************************
** NAME:        SockAdrPortOut                              NETMON
** SYNOPSIS:    void SockAdrPortOut
**              (
**                struct in_addr *inadr ,
**                u_short         inport
**              )
** RELATED TO:  TcpSo..(), Udpso..()
** DESCRIPTION: display socket_in address and its port
** REMARKS:     - recognizes wildcard numbers
** RETURN TYPE: none
** EXIT STATUS: -
*****************************************************************/

void SockAdrPortOut( struct in_addr *inadr ,u_short inport, int proto )
{
    u_long  addr;
    u_short port;

    addr = ntohl( inadr->s_addr );
    port = ntohs( inport );

    if ( addr == INADDR_ANY )
        cprintf( "              *," );
    else
#ifndef RESOLVER
        printf( "%15.15s,", inet_ntoa( inadr ));
#else
        printf( "%15.15s,", AddrLookup( addr ));
#endif
    if ( port == 0 )
        cprintf( "*        " );
    else
#ifndef RESOLVER
        cprintf( "%- 7.7d  ", port );
#else
        cprintf( "%7.7s  ", PortLookup( inport, proto ));
#endif
}

/*****************************************************************
** NAME:        SockStateInfo                               NETMON
** SYNOPSIS:    void SockStateInfo
**              (
**                short state
**              )
** RELATED TO:  TcpSo..(), Udpso..()
** DESCRIPTION: display socket state flags
** REMARKS:     -
** RETURN TYPE: none
** EXIT STATUS: -
*****************************************************************/

void  SockStateInfo( short state )
{
    char buf[ 32 ] = "________________";

    if ( state & SS_NOFDREF )
         buf[ 15 ] = 'N';
    if ( state & SS_ISCONNECTED )
         buf[ 14 ] = 'R';
    if ( state & SS_ISCONNECTING )
         buf[ 13 ] = 'C';
    if ( state & SS_ISDISCONNECTING )
         buf[ 12 ] = 'D';
    if ( state & SS_CANTSENDMORE )
         buf[ 11 ] = 'S';
    if ( state & SS_CANTRCVMORE )
         buf[ 10 ] = 'R';
    if ( state & SS_RCVATMARK )
         buf[  9 ] = 'M';
    if ( state & SS_PRIV )
         buf[  8 ] = 'P';
    if ( state & SS_NBIO )
         buf[  7 ] = 'N';
    if ( state & SS_ASYNC )
         buf[  6 ] = 'A';
    if ( state & SS_PIPE )
         buf[  5 ] = 'P';

    cprintf( "%16s  ", buf );
}

/*****************************************************************
** NAME:        SockOptions                                 NETMON
** SYNOPSIS:    void SockOptions
**              (
**                SOCKET *pso
**              )
** RELATED TO:  TcpSo..(), Udpso..()
** DESCRIPTION: display socket options flags, cq. settings
** REMARKS:     -
** RETURN TYPE: none
** EXIT STATUS: -
*****************************************************************/

void SockOptions( SOCKET *pso )
{
    short option = pso->so_options;

    char buf[ 17 ] = "________________";

    if ( option & SO_DEBUG )
         buf[ 15 ] = 'D';
    if ( option & SO_ACCEPTCONN )
         buf[ 14 ] = 'A';
    if ( option & SO_REUSEADDR )
         buf[ 13 ] = 'R';
    if ( option & SO_KEEPALIVE )
         buf[ 12 ] = 'K';
    if ( option & SO_DONTROUTE )
         buf[ 11 ] = 'r';
    if ( option & SO_BROADCAST )
         buf[ 10 ] = 'B';
    if ( option & SO_USELOOPBACK )
         buf[  9 ] = 'L';
    if ( option & SO_LINGER )
         buf[  8 ] = 'l';
    if ( option & SO_OOBINLINE )
         buf[  7 ] = 'O';

    cprintf( "%16s  ", buf );
}

/*****************************************************************
** NAME:        SockBufSize                                 NETMON
** SYNOPSIS:    void SockBufSize
**              (
**                SOCKET *pso
**              )
** RELATED TO:  TcpSo..(), Udpso..()
** DESCRIPTION: display socket buffer sizes
** REMARKS:     -
** RETURN TYPE: none
** EXIT STATUS: -
*****************************************************************/

void SockBufSize( SOCKET *pso )
{
    cprintf( "% 6ld  %6ld  ", pso->so_rcv.sb_mbmax, pso->so_snd.sb_mbmax );
    cprintf( "% 6ld  %6ld  ", pso->so_rcv.sb_cc,    pso->so_snd.sb_cc );
}

/* end of nettcpso.c */




